unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls,
  Forms, Dialogs, DirectXGraphics, DX8GFX, D3DX8;

type
  TForm1 = class(TForm)
    procedure AppOnIdle(Sender: TObject; var Done: Boolean);
    procedure AppOnActivate(Sender: TObject);
    procedure AppOnDeactivate(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

  FRAMECONTROL : dWord;
  DXReady      : boolean = false;
  WindowReady  : boolean = false;

  VertexBuffer : IDirect3DVertexBuffer8;

  // Matrices
  matWorld : TD3DXMatrix; // World
  matView  : TD3DXMatrix; // View
  matProj  : TD3DXMatrix; // Projection

type
  // The structure of the custom vertex.
  PCustomVertex = ^TCustomVertex;
  TCustomVertex = record
                    X, Y, Z : single; // Position for the vertex.
                    DColor  : dWord;  // Diffuse color.
                  end;

const
  // Flexible vector format (FVF) description.
  // Describes the custom vertex structure (TCustomVertex).
  D3DFVF_CUSTOMVERTEX = D3DFVF_XYZ or D3DFVF_DIFFUSE;

const
  WRN00 = 'Error initializing 3D Hardware.'#10+
          'Now software will be used...';
  ERR00 = 'Could not initialize DirectX Graphics.'#10+
          'You must have installed DirectX 8 or better '#10+
          'and a 16, 24 or 32 bpp desktop to run this sample.';
  ERR01 = 'Error initializing Vertex Buffer.';

  FRAMEAT = 15; // Set to 0 to full speed.

implementation

{$R *.DFM}

// Tutorial 3 - Steps 1, 2 and 3 *************************************

function GetD3DXVector3(_x, _y, _z: single): TD3DXVector3;

begin
  Result.x := _x;
  Result.y := _y;
  Result.z := _z;
end;

// -------------------------------------------------------------------

procedure SetupMatrices;

var _v1, _v2, _v3 : TD3DXVector3;

begin
  // World.
  D3DXMatrixRotationY(matWorld, sin(GetTickCount/500) * 0.4);
  D3DDEV8.SetTransform(D3DTS_WORLD, matWorld);

  // View.
  _v1 := GetD3DXVector3( 0.0, 0.0, -5.0);
  _v2 := GetD3DXVector3( 0.0, 0.0,  0.0);
  _v3 := GetD3DXVector3( 0.0, 1.0,  0.0);
  D3DXMatrixLookAtLH(matView, _v1, _v2, _v3);
  D3DDEV8.SetTransform(D3DTS_VIEW, matView);

  // Projection.
  D3DXMatrixPerspectiveFovLH(matProj, PI/4, 1.0, 1.0, 100.0);
  D3DDEV8.SetTransform(D3DTS_PROJECTION, matProj);
end;

// *******************************************************************

procedure RenderFrame;

begin
  // Clear the back buffer to a black color.
  D3DDEV8.Clear(0, NIL, D3DCLEAR_TARGET, $FF000000, 1.0, 0);

  // Begin the scene.
  D3DDEV8.BeginScene;

  // Set Transform states
  SetupMatrices;

  // Set the stream source.
  D3DDEV8.SetStreamSource( 0, VertexBuffer, sizeof(TCustomVertex));

  // Set the vertex shader.
  D3DDEV8.SetVertexShader(D3DFVF_CUSTOMVERTEX);

  // Render the vertices.
  D3DDEV8.DrawPrimitive(D3DPT_TRIANGLELIST, 0, 1);

  // End the scene.
  D3DDEV8.EndScene;

  // Display scene.
  D3DDEV8.Present(NIL, NIL, 0, NIL);
end;

// -------------------------------------------------------------------

procedure TForm1.AppOnIdle(Sender: TObject; var Done: Boolean);

begin
  Done := false;
  if (not DXReady) or (not WindowReady) then exit;
  RenderFrame;
  // Speed control
  while GetTickCount - FRAMECONTROL < FRAMEAT do Sleep(0);
  FRAMECONTROL := GetTickCount;
end;

// ------------------------------------------------------------------

procedure TForm1.AppOnActivate(Sender: TObject);

begin
  WindowReady := true;
end;

// ------------------------------------------------------------------

procedure TForm1.AppOnDeactivate(Sender: TObject);

begin
  WindowReady := false;
end;

// -------------------------------------------------------------------

procedure hHalt(_errMsg: pChar);

begin
  MessageBox(0, _errMsg, 'Error', MB_ICONERROR);
  Application.Terminate;
end;

// -------------------------------------------------------------------

function GetCustomVertex(_x, _y, _z: single;
                         _dcolor: dWord): TCustomVertex;

begin
  Result.X := _x;
  Result.Y := _y;
  Result.Z := _z;
  Result.DColor := _dcolor;
end;

// -------------------------------------------------------------------

function InitVertexBuffer: boolean;

var _vb  : array[0..2] of TCustomVertex;
    _ptr : pByte;

begin
  Result := false;

  // Initialize the vertices.
  _vb[0] := GetCustomVertex(-1, -1, 0.0, $FFFF0000);
  _vb[1] := GetCustomVertex( 1, -1, 0.0, $FF00FF00);
  _vb[2] := GetCustomVertex( 0,  1, 0.0, $FF0000FF);

  // Create a vertex buffer.
  if failed(D3DDEV8.CreateVertexBuffer(SizeOf(_vb), 0,
                                       D3DFVF_CUSTOMVERTEX,
                                       D3DPOOL_DEFAULT,
                                       VertexBuffer)) then exit;

  // Lock, fill and unlock vertex buffer.
  if failed(VertexBuffer.Lock(0, SizeOf(_vb), _ptr, 0)) then exit;
  Move(_vb, _ptr^, SizeOf(_vb));
  VertexBuffer.Unlock;

  // All right
  Result := true;
end;

// -------------------------------------------------------------------

procedure TForm1.FormActivate(Sender: TObject);

begin
  FRAMECONTROL := GetTickCount;

  // Initialize application events
  Application.OnIdle       := AppOnIdle;
  Application.OnActivate   := AppOnActivate;
  Application.OnDeactivate := AppOnDeactivate;
  Application.OnRestore    := AppOnActivate;
  Application.OnMinimize   := AppOnDeActivate;

  // Initialize Direct Graphics (first try with Hardware Accel. (HAL))
  if not InitDGfx(Handle, false, 0, 0, 0, 0, true, false) then
    begin
      MessageBox(0, WRN00, 'Warning.', MB_ICONWARNING);
      CloseDGfx;
      if not InitDGfx(Handle, false, 0, 0, 0, 0, false, false) then
        begin hHalt(ERR00); exit; end;
    end;

  // Initialize VertexBuffer
  if not InitVertexBuffer then begin hHalt(ERR01); exit; end;

  // Tutorial 3 - Step 0 **********************************************

  // Turn off culling, so we see the front and back of the triangle
  D3DDEV8.SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);

  // Turn off D3D lighting, since we are providing our own vertex colors
  D3DDEV8.SetRenderState(D3DRS_LIGHTING, Ord(false));

  // ******************************************************************

  // All ok
  DXReady := true;
end;

// -------------------------------------------------------------------

procedure TForm1.FormDestroy(Sender: TObject);

begin
  DXReady := false;
  // Free Vertex Buffer
  VertexBuffer := NIL;
  // Free DX
  CloseDGfx;
end;

// -------------------------------------------------------------------

end.
