unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls,
  Forms, Dialogs, DirectXGraphics, DX8GFX, D3DX8;

type
  TForm1 = class(TForm)
    procedure AppOnIdle(Sender: TObject; var Done: Boolean);
    procedure AppOnActivate(Sender: TObject);
    procedure AppOnDeactivate(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

  FRAMECONTROL : dWord;
  DXReady      : boolean = false;
  WindowReady  : boolean = false;

  VertexBuffer : IDirect3DVertexBuffer8;

  // Matrices
  matWorld : TD3DXMatrix; // World
  matView  : TD3DXMatrix; // View
  matProj  : TD3DXMatrix; // Projection

type
  // The structure of the custom vertex.
  PCustomVertex = ^TCustomVertex;
  TCustomVertex = record
                    Position : TD3DXVector3; // Position
                    Normal   : TD3DXVector3; // Surface Normal
                  end;

const
  // Flexible vector format (FVF) description.
  // Describes the custom vertex structure (TCustomVertex).
  D3DFVF_CUSTOMVERTEX = D3DFVF_XYZ or D3DFVF_NORMAL;

const
  WRN00 = 'Error initializing 3D Hardware.'#10+
          'Now software will be used...';
  ERR00 = 'Could not initialize DirectX Graphics.'#10+
          'You must have installed DirectX 8 or better '#10+
          'and a 16, 24 or 32 bpp desktop to run this sample.';
  ERR01 = 'Error initializing Vertex Buffer.';

  FRAMEAT = 15; // Set to 0 to full speed.

implementation

{$R *.DFM}

// Tutorial 4 - Step 2 ***********************************************

procedure SetupMaterialAndLight;

var _mat : TD3DMaterial8;
    _lit : TD3DLight8;
    _tim : single;
    _vecDir : TD3DXVector3;

begin
  // Set Material
  FillChar(_mat, sizeof(_mat), 0 );
  with _mat do
    begin
      Ambient.r := 1.0;
      Ambient.g := 1.0;
      Ambient.b := 0.0;
      Ambient.a := 1.0;
      Diffuse := Ambient;
      D3DDEV8.SetMaterial(_mat);
    end;

  // Set Directional Light
  FillChar(_lit, sizeof(_lit), 0 );
  with _lit do
    begin
      _Type := D3DLIGHT_DIRECTIONAL;
      Diffuse.r := 1.0;
      Diffuse.g := 1.0;
      Diffuse.b := 1.0;
      _tim := GetTickCount / 360;
      _vecDir := D3DXVECTOR3(cos(_tim), 0.0, sin(_tim));
      D3DXVec3Normalize(Direction, _vecDir);
      Range   := 1000.0;
      D3DDEV8.SetLight(0, _lit);
      D3DDEV8.LightEnable(0, true);
    end;
end;

// *******************************************************************

function GetD3DXVector3(_x, _y, _z: single): TD3DXVector3;

begin
  Result.x := _x;
  Result.y := _y;
  Result.z := _z;
end;

// -------------------------------------------------------------------

procedure SetupMatrices;

var _v1, _v2, _v3 : TD3DXVector3;

begin
  // World.
  D3DXMatrixRotationX(matWorld, sin(GetTickCount/500) * 0.4);
  D3DDEV8.SetTransform(D3DTS_WORLD, matWorld);

  // View.
  _v1 := GetD3DXVector3( 0.0, 0.0, -5.0);
  _v2 := GetD3DXVector3( 0.0, 0.0,  0.0);
  _v3 := GetD3DXVector3( 0.0, 1.0,  0.0);
  D3DXMatrixLookAtLH(matView, _v1, _v2, _v3);
  D3DDEV8.SetTransform(D3DTS_VIEW, matView);

  // Projection.
  D3DXMatrixPerspectiveFovLH(matProj, PI/4, 1.0, 1.0, 100.0);
  D3DDEV8.SetTransform(D3DTS_PROJECTION, matProj);
end;

// -------------------------------------------------------------------

procedure RenderFrame;

begin
  // Clear the back buffer to a black color.
  D3DDEV8.Clear(0, NIL, D3DCLEAR_TARGET or D3DCLEAR_ZBUFFER,
                $FF000000, 1.0, 0);

  // Begin the scene.
  D3DDEV8.BeginScene;

  // Set Material and Light.
  SetupMaterialAndLight;

  // Set Transform states.
  SetupMatrices;

  // Set the stream source.
  D3DDEV8.SetStreamSource(0, VertexBuffer, sizeof(TCustomVertex));

  // Set the vertex shader.
  D3DDEV8.SetVertexShader(D3DFVF_CUSTOMVERTEX);

  // Render the vertices.
  D3DDEV8.DrawPrimitive(D3DPT_TRIANGLESTRIP, 0, 98);

  // End the scene.
  D3DDEV8.EndScene;

  // Display scene.
  D3DDEV8.Present(NIL, NIL, 0, NIL);
end;

// -------------------------------------------------------------------

procedure TForm1.AppOnIdle(Sender: TObject; var Done: Boolean);

begin
  Done := false;
  if (not DXReady) or (not WindowReady) then exit;
  RenderFrame;
  // Speed control
  while GetTickCount - FRAMECONTROL < FRAMEAT do Sleep(0);
  FRAMECONTROL := GetTickCount;
end;

// ------------------------------------------------------------------

procedure TForm1.AppOnActivate(Sender: TObject);

begin
  WindowReady := true;
end;

// ------------------------------------------------------------------

procedure TForm1.AppOnDeactivate(Sender: TObject);

begin
  WindowReady := false;
end;

// -------------------------------------------------------------------

procedure hHalt(_errMsg: pChar);

begin
  MessageBox(0, _errMsg, 'Error', MB_ICONERROR);
  Application.Terminate;
end;

// Tutorial 4 - Step 1 ***********************************************

function GetCustomVertex(_px, _py, _pz,
                         _nx, _ny, _nz: single): TCustomVertex;

begin
  with Result do
   begin
     Position.x := _px;
     Position.y := _py;
     Position.z := _pz;
     Normal.x := _nx;
     Normal.y := _ny;
     Normal.z := _nz;
   end;
end;

// -------------------------------------------------------------------

function InitVertexBuffer: boolean;

var _vb  : array[0..99] of TCustomVertex;
    _cnt : longint;
    _ptr : pByte;
    _theta : single;

begin
  Result := false;

  // Initialize the vertices.
  for _cnt := 0 to 49 do
    begin
      _theta := (2 * PI * _cnt) /49;
      _vb[2 * _cnt] := GetCustomVertex(sin(_theta), -1.0, cos(_theta),
                                       sin(_theta),  0.0, cos(_theta));
      _vb[2 * _cnt + 1] := GetCustomVertex(sin(_theta),  1.0, cos(_theta),
                                           sin(_theta),  0.0, cos(_theta));
    end;

  // Create a vertex buffer.
  if failed(D3DDEV8.CreateVertexBuffer(SizeOf(_vb), 0,
                                       D3DFVF_CUSTOMVERTEX,
                                       D3DPOOL_DEFAULT,
                                       VertexBuffer)) then exit;

  // Lock, fill and unlock vertex buffer.
  if failed(VertexBuffer.Lock(0, SizeOf(_vb), _ptr, 0)) then exit;
  Move(_vb, _ptr^, SizeOf(_vb));
  VertexBuffer.Unlock;

  // All right
  Result := true;
end;

// *******************************************************************

procedure TForm1.FormActivate(Sender: TObject);

begin
  FRAMECONTROL := GetTickCount;

  // Initialize application events
  Application.OnIdle       := AppOnIdle;
  Application.OnActivate   := AppOnActivate;
  Application.OnDeactivate := AppOnDeactivate;
  Application.OnRestore    := AppOnActivate;
  Application.OnMinimize   := AppOnDeActivate;

  // Initialize Direct Graphics (first try with Hardware Accel. (HAL))
  if not InitDGfx(Handle, false, 0, 0, 0, 0, true, true) then
    begin
      MessageBox(0, WRN00, 'Warning.', MB_ICONWARNING);
      CloseDGfx;
      if not InitDGfx(Handle, false, 0, 0, 0, 0, false, true) then
        begin hHalt(ERR00); exit; end;
    end;

  // Initialize VertexBuffer
  if not InitVertexBuffer then begin hHalt(ERR01); exit; end;

  // Set Material And Light
  SetupMaterialAndLight;

  // Tutorial 4 - Step 0 **********************************************

  // Disable culling.
  D3DDEV8.SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);

  // Enable lights.
  D3DDEV8.SetRenderState(D3DRS_LIGHTING, 1);

  // Set ambiental light.
  D3DDEV8.SetRenderState(D3DRS_AMBIENT, $404040);

  // ******************************************************************

  // All ok
  DXReady := true;
end;

// -------------------------------------------------------------------

procedure TForm1.FormDestroy(Sender: TObject);

begin
  DXReady := false;
  // Free Vertex Buffer
  VertexBuffer := NIL;
  // Free DX
  CloseDGfx;
end;

// -------------------------------------------------------------------

end.
