unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls,
  Forms, Dialogs, DirectXGraphics, DX8GFX, D3DX8;

type
  TForm1 = class(TForm)
    procedure AppOnIdle(Sender: TObject; var Done: Boolean);
    procedure AppOnActivate(Sender: TObject);
    procedure AppOnDeactivate(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

  FRAMECONTROL : dWord;
  DXReady      : boolean = false;
  WindowReady  : boolean = false;

  VertexBuffer : IDirect3DVertexBuffer8;
  Texture      : IDirect3DTexture8;

  // Matrices
  matWorld : TD3DXMatrix; // World
  matView  : TD3DXMatrix; // View
  matProj  : TD3DXMatrix; // Projection

type
  // The structure of the custom vertex.
  PCustomVertex = ^TCustomVertex;
  TCustomVertex = record
                    Position : TD3DXVector3; // Position.
                    DColor   : TD3DColor;    // Diffuse Color.
                    U, V     : single;       // Texture coordinates.
                  end;

const
  // Flexible vector format (FVF) description.
  // Describes the custom vertex structure (TCustomVertex).
  D3DFVF_CUSTOMVERTEX = D3DFVF_XYZ or D3DFVF_DIFFUSE or D3DFVF_TEX1;

  // Automatic texture coordinate generation.
  SHOW_HOW_TO_USE_TCI   = false;
  // Texture filtering.
  USE_TEXTURE_FILTERING = true;

const
  TEXTFILENAME = 'CLOUDS.PNG';
  WRN00 = 'Error initializing 3D Hardware.'#10+
          'Now software will be used...';
  ERR00 = 'Could not initialize DirectX Graphics.'#10+
          'You must have installed DirectX 8 or better '#10+
          'and a 16, 24 or 32 bpp desktop to run this sample.';
  ERR01 = 'Error initializing Vertex Buffer.';
  ERR02 = 'Error loading file: ';

  FRAMEAT = 15; // Set to 0 to full speed.

implementation

{$R *.DFM}

// -------------------------------------------------------------------

function GetD3DXVector3(_x, _y, _z: single): TD3DXVector3;

begin
  Result.x := _x;
  Result.y := _y;
  Result.z := _z;
end;

// -------------------------------------------------------------------

procedure SetupMatrices;

var _v1, _v2, _v3 : TD3DXVector3;

begin
  // World.
  D3DXMatrixRotationX(matWorld, sin(GetTickCount/1000) * 1.0);
  D3DDEV8.SetTransform(D3DTS_WORLD, matWorld);

  // View.
  _v1 := GetD3DXVector3( 0.0, 0.0, -5.0);
  _v2 := GetD3DXVector3( 0.0, 0.0,  0.0);
  _v3 := GetD3DXVector3( 0.0, 1.0,  0.0);
  D3DXMatrixLookAtLH(matView, _v1, _v2, _v3);
  D3DDEV8.SetTransform(D3DTS_VIEW, matView);

  // Projection.
  D3DXMatrixPerspectiveFovLH(matProj, PI/4, 1.0, 1.0, 100.0);
  D3DDEV8.SetTransform(D3DTS_PROJECTION, matProj);
end;

// -------------------------------------------------------------------

procedure RenderFrame;

var _mat : TD3DXMatrix;

begin
  // Clear the back buffer to a black color.
  D3DDEV8.Clear(0, NIL, D3DCLEAR_TARGET or D3DCLEAR_ZBUFFER,
                $FF000000, 1.0, 0);

  // Begin the scene.
  D3DDEV8.BeginScene;

  // Set Transform states.
  SetupMatrices;

  // TCI (texture coordinate index)
  if SHOW_HOW_TO_USE_TCI then
    begin
      with _mat do
        begin
          _11 := 0.25; _12 := 0.00; _13 := 0.00; _14 := 0.00;
          _21 := 0.00; _22 :=-0.25; _23 := 0.00; _24 := 0.00;
          _31 := 0.00; _32 := 0.00; _33 := 1.00; _34 := 0.00;
          _41 := 0.50; _42 := 0.50; _43 := 0.00; _44 := 1.00;
       end;

      D3DDEV8.SetTransform( D3DTS_TEXTURE0, _mat );
      D3DDEV8.SetTextureStageState( 0, D3DTSS_TEXTURETRANSFORMFLAGS, D3DTTFF_COUNT2 );
      D3DDEV8.SetTextureStageState( 0, D3DTSS_TEXCOORDINDEX, D3DTSS_TCI_CAMERASPACEPOSITION );
   end;

  // Set the stream source.
  D3DDEV8.SetStreamSource(0, VertexBuffer, sizeof(TCustomVertex));

  // Set the vertex shader.
  D3DDEV8.SetVertexShader(D3DFVF_CUSTOMVERTEX);

  // Render the vertices.
  D3DDEV8.DrawPrimitive(D3DPT_TRIANGLESTRIP, 0, 98);

  // End the scene.
  D3DDEV8.EndScene;

  // Display scene.
  D3DDEV8.Present(NIL, NIL, 0, NIL);
end;

// -------------------------------------------------------------------

procedure TForm1.AppOnIdle(Sender: TObject; var Done: Boolean);

begin
  Done := false;
  if (not DXReady) or (not WindowReady) then exit;
  RenderFrame;
  // Speed control
  while GetTickCount - FRAMECONTROL < FRAMEAT do Sleep(0);
  FRAMECONTROL := GetTickCount;
end;

// ------------------------------------------------------------------

procedure TForm1.AppOnActivate(Sender: TObject);

begin
  WindowReady := true;
end;

// ------------------------------------------------------------------

procedure TForm1.AppOnDeactivate(Sender: TObject);

begin
  WindowReady := false;
end;

// -------------------------------------------------------------------

procedure hHalt(_errMsg: pChar);

begin
  MessageBox(0, _errMsg, 'Error', MB_ICONERROR);
  Application.Terminate;
end;

// -------------------------------------------------------------------

function GetCustomVertex(_px, _py, _pz, _u, _v: single;
                         _dcolor: dWord): TCustomVertex;

begin
  with Result do
   begin
     Position.x := _px;
     Position.y := _py;
     Position.z := _pz;
     U := _u;
     V := _v;
     DColor := _dcolor;
   end;
end;

// -------------------------------------------------------------------

function InitVertexBuffer: boolean;

var _vb  : array[0..99] of TCustomVertex;
    _cnt : longint;
    _ptr : pByte;
    _theta : single;

begin
  Result := false;

  // Tutorial 5 - Step 2 **********************************************

  // Initialize the vertices.
  for _cnt := 0 to 49 do
    begin
      _theta := (2 * PI * _cnt) /49;
      _vb[2 * _cnt] := GetCustomVertex(sin(_theta), -1.0, cos(_theta),
                                       _cnt/49, 1.0, $FFFF7F00);
      _vb[2 * _cnt + 1] := GetCustomVertex(sin(_theta),  1.0, cos(_theta),
                                           _cnt/49, 0.0, $FFFFFFFF);
    end;

  // ******************************************************************

  // Create a vertex buffer.
  if failed(D3DDEV8.CreateVertexBuffer(SizeOf(_vb), 0,
                                       D3DFVF_CUSTOMVERTEX,
                                       D3DPOOL_DEFAULT,
                                       VertexBuffer)) then exit;

  // Lock, fill and unlock vertex buffer.
  if failed(VertexBuffer.Lock(0, SizeOf(_vb), _ptr, 0)) then exit;
  Move(_vb, _ptr^, SizeOf(_vb));
  VertexBuffer.Unlock;

  // All right
  Result := true;
end;

// -------------------------------------------------------------------

procedure TForm1.FormActivate(Sender: TObject);

begin
  FRAMECONTROL := GetTickCount;

  // Initialize application events
  Application.OnIdle       := AppOnIdle;
  Application.OnActivate   := AppOnActivate;
  Application.OnDeactivate := AppOnDeactivate;
  Application.OnRestore    := AppOnActivate;
  Application.OnMinimize   := AppOnDeActivate;

  // Initialize Direct Graphics (first try with Hardware Accel. (HAL))
  if not InitDGfx(Handle, false, 0, 0, 0, 0, true, true) then
    begin
      MessageBox(0, WRN00, 'Warning.', MB_ICONWARNING);
      CloseDGfx;
      if not InitDGfx(Handle, false, 0, 0, 0, 0, false, true) then
        begin hHalt(ERR00); exit; end;
    end;

  // Tutorial 5 - Step 2 **********************************************

  // Load Texture.
  if failed(D3DXCreateTextureFromFileA(D3DDEV8, TEXTFILENAME, Texture)) then
    begin hHalt(ERR02 + TEXTFILENAME); exit; end;

  // ******************************************************************

  // Initialize VertexBuffer
  if not InitVertexBuffer then begin hHalt(ERR01); exit; end;

  // Disable culling.
  D3DDEV8.SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);

  // Disable lights.
  D3DDEV8.SetRenderState(D3DRS_LIGHTING, 0);

  // Set ambiental light.
  D3DDEV8.SetRenderState(D3DRS_AMBIENT, $404040);

  // Set texture filtering
  if USE_TEXTURE_FILTERING then
    begin
      D3DDEV8.SetTextureStageState(0, D3DTSS_MAGFILTER, ord(D3DTEXF_LINEAR));
      D3DDEV8.SetTextureStageState(0, D3DTSS_MINFILTER, ord(D3DTEXF_LINEAR));
    end;

  // Tutorial 5 - Step 3 **********************************************

  // Set the texture that the will be used to render.
  D3DDEV8.SetTexture(0, Texture);

  // Set the texture stage state values.
  D3DDEV8.SetTextureStageState( 0, D3DTSS_COLOROP,   D3DTOP_MODULATE );
  D3DDEV8.SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
  D3DDEV8.SetTextureStageState( 0, D3DTSS_COLORARG2, D3DTA_DIFFUSE );
  D3DDEV8.SetTextureStageState( 0, D3DTSS_ALPHAOP,   D3DTOP_DISABLE );

  // ******************************************************************

  // All ok
  DXReady := true;
end;

// -------------------------------------------------------------------

procedure TForm1.FormDestroy(Sender: TObject);

begin
  DXReady := false;
  VertexBuffer := NIL; // Free Vertex Buffer
  Texture := NIL; // Free Texture;
  CloseDGfx; // Free DX
end;

// -------------------------------------------------------------------

end.
