unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls,
  Forms, Dialogs, DirectXGraphics, DX8GFX, D3DX8;

type
  TForm1 = class(TForm)
    procedure AppOnIdle(Sender: TObject; var Done: Boolean);
    procedure AppOnActivate(Sender: TObject);
    procedure AppOnDeactivate(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

  FRAMECONTROL : dWord;
  DXReady      : boolean = false;
  WindowReady  : boolean = false;

  VertexBuffer1 : IDirect3DVertexBuffer8;
  VertexBuffer2 : IDirect3DVertexBuffer8;

  NumPickedTriangles : dWord;
  PickT, PickU, PickV : single;

  // Matrices
  matWorld : TD3DXMatrix; // World
  matView  : TD3DXMatrix; // View
  matProj  : TD3DXMatrix; // Projection

type
  // The structure of the custom vertex.
  PCustomVertex = ^TCustomVertex;
  TCustomVertex = record
                    Position : TD3DXVector3; // Position
                  end;
  PCV = ^TCV;
  TCV = array[0..0] of TCustomVertex;

const
  // Flexible vector format (FVF) description.
  // Describes the custom vertex structure (TCustomVertex).
  D3DFVF_CUSTOMVERTEX = D3DFVF_XYZ;

  // Number of faces
  FACENUM = 4;

  // Display resolution
  DWIDTH  = 400;
  DHEIGHT = 300;

const
  WRN00 = 'Error initializing 3D Hardware.'#10+
          'Now software will be used...';
  ERR00 = 'Could not initialize DirectX Graphics.'#10+
          'You must have installed DirectX 8 or better '#10+
          'and a 16, 24 or 32 bpp desktop to run this sample.';
  ERR01 = 'Error initializing Vertex Buffer.';

  FRAMEAT = 15; // Set to 0 to full speed.

implementation

{$R *.DFM}

// -------------------------------------------------------------------

procedure SetMaterial;

var _mat : TD3DMaterial8;

begin
  // Set Material
  FillChar(_mat, sizeof(_mat), 0 );
  with _mat do
    begin
      Ambient.r := 1.0;
      Ambient.g := 1.0;
      Ambient.b := 0.0;
      Ambient.a := 1.0;
      D3DDEV8.SetMaterial(_mat);
    end;
end;

// -------------------------------------------------------------------

function GetD3DXVector3(_x, _y, _z: single): TD3DXVector3;

begin
  Result.x := _x;
  Result.y := _y;
  Result.z := _z;
end;

// -------------------------------------------------------------------

procedure SetupMatrices;

var _v1, _v2, _v3 : TD3DXVector3;
    _time : dWord;

begin
  // World.
  D3DXMatrixRotationX(matWorld, 0.0);
  D3DDEV8.SetTransform(D3DTS_WORLD, matWorld);

  // Rotate the camera
  _time := GetTickCount;
  _v2 := GetD3DXVector3(0.0, 0.0, 0.0);
  _v3 := GetD3DXVector3(0.0, 1.0, 0.0);
  _v1.x := -cos(_time/3000.0) * 4.0;
  _v1.y := 1.0;
  _v1.z :=  sin(_time/3000.0) * 4.0;

  // View.
  D3DXMatrixLookAtLH(matView, _v1, _v2, _v3);
  D3DDEV8.SetTransform(D3DTS_VIEW, matView);

  // Projection.
  D3DXMatrixPerspectiveFovLH(matProj, PI/4, 1.0, 1.0, 100.0);
  D3DDEV8.SetTransform(D3DTS_PROJECTION, matProj);
end;

// -------------------------------------------------------------------
// Name: IntersectTriangle()
// Desc: Given a ray origin (orig) and direction (dir), and three
//       vertices of of a triangle, this function returns TRUE and
//       the interpolated texture coordinates if the ray intersects
//       the triangle.
// -------------------------------------------------------------------

function IntersectTriangle(_orig, _dir, _v0, _v1, _v2 : TD3DXVector3;
                           var _t, _u, _v: single): boolean;

var _edge1, _edge2 : TD3DXVector3;
    _pvec, _tvec, _qvec : TD3DXVector3;
    _det, _invDet : single;

begin
  Result := false;

  // Find vectors for two edges sharing vert0
  _edge1.x := _v1.x - _v0.x;
  _edge1.y := _v1.y - _v0.y;
  _edge1.z := _v1.z - _v0.z;

  _edge2.x := _v2.x - _v0.x;
  _edge2.y := _v2.y - _v0.y;
  _edge2.z := _v2.z - _v0.z;

  // Begin calculating determinant - also used to calculate U parameter
  D3DXVec3Cross(_pvec, _dir, _edge2);

  // If determinant is near zero, ray lies in plane of triangle
  _det := D3DXVec3Dot(_edge1, _pvec);
  if (_det < 0.0001) then exit;

  // Calculate distance from vert0 to ray origin
  _tvec.x := _orig.x - _v0.x;
  _tvec.y := _orig.y - _v0.y;
  _tvec.z := _orig.z - _v0.z;

  // Calculate U parameter and test bounds
  _u := D3DXVec3Dot(_tvec, _pvec);
  if (_u < 0.0) or (_u > _det) then exit;

  // Prepare to test V parameter
  D3DXVec3Cross(_qvec, _tvec, _edge1);

  // Calculate V parameter and test bounds
  _v := D3DXVec3Dot(_dir, _qvec);
  if (_v < 0.0) or (_u + _v > _det) then exit;

  // Calculate t, scale parameters, ray intersects triangle
  _t := D3DXVec3Dot(_edge2, _qvec);
  _invDet := 1.0 / _det;
  _t := _t * _invDet;
  _u := _u * _invDet;
  _v := _v * _invDet;

  Result := true;
end;

// -------------------------------------------------------------------

procedure Pick(_hWnd: hWnd);

var _rayDir, _rayOrig : TD3DXVector3;
    _matProj, _matView, _m : TD3DXMatrix;
    _cursor : TPoint;
    _v : TD3DXVector3;
    _p, _p2 : PCV;
    _i : dWord;

    _v0, _v1, _v2 : TD3DXVector3;

begin
  NumPickedTriangles := 0;

  // Get the pick ray from the mouse position.
  if GetCapture <> 0 then
    begin
      D3DDEV8.GetTransform(D3DTS_PROJECTION, _matProj);

      GetCursorPos(_cursor);
      ScreenToClient(_hWnd, _cursor);

      // Compute the vector of the pick ray in screen space
      _v.x :=  (((2.0 * _cursor.x) / DWIDTH) - 1) / _matProj._11;
      _v.y := -(((2.0 * _cursor.y) / DHEIGHT) - 1) / _matProj._22;
      _v.z := 1.0;

      // Get the inverse view matrix
      D3DDEV8.GetTransform(D3DTS_VIEW, _matView);
      D3DXMatrixInverse(_m, NIL, _matView);

      // Transform the screen space pick ray into 3D space
      _rayDir.x  := _v.x*_m._11 + _v.y*_m._21 + _v.z*_m._31;
      _rayDir.y  := _v.x*_m._12 + _v.y*_m._22 + _v.z*_m._32;
      _rayDir.z  := _v.x*_m._13 + _v.y*_m._23 + _v.z*_m._33;
      _rayOrig.x := _m._41;
      _rayOrig.y := _m._42;
      _rayOrig.z := _m._43;
    end;


  // Get the picked triangle
  if GetCapture <> 0 then
    begin
      VertexBuffer1.Lock(0, 0, pByte(_p), 0);
      VertexBuffer2.Lock(0, 0, pByte(_p2), 0);

      for _i := 0 to FACENUM - 1 do
        begin
          _v0 := _p[_i*3+0].Position;
          _v1 := _p[_i*3+1].Position;
          _v2 := _p[_i*3+2].Position;

          // Check if the pick ray passes through this point
          if IntersectTriangle(_rayOrig, _rayDir, _v0, _v1, _v2, PickT, PickU, PickV) then
            begin
              _p2[NumPickedTriangles*3+0].Position := _v0;
              _p2[NumPickedTriangles*3+1].Position := _v1;
              _p2[NumPickedTriangles*3+2].Position := _v2;
              inc(NumPickedTriangles);
            end;
        end;

      VertexBuffer2.Unlock;
      VertexBuffer1.Unlock;
    end;
end;

// -------------------------------------------------------------------

procedure RenderFrame(_hWnd: hWnd);

var _cur : TPoint;

begin
  // Set up the cursor
  GetCursorPos(_cur);
  ScreenToClient(_hWnd, _cur);
  D3DDEV8.SetCursorPosition(_cur.x, _cur.y, 0);

  // Check for picked triangles
  Pick(_hWnd);

  // Clear the back buffer to a black color.
  D3DDEV8.Clear(0, NIL, D3DCLEAR_TARGET or D3DCLEAR_ZBUFFER,
                $FF000000, 1.0, 0);

  // Begin the scene.
  D3DDEV8.BeginScene;

  // Set Transform states.
  SetupMatrices;

  // Set the vertex shader.
  D3DDEV8.SetVertexShader(D3DFVF_CUSTOMVERTEX);

  // Set the stream source.
  D3DDEV8.SetStreamSource(0, VertexBuffer1, sizeof(TCustomVertex));

  // Render the vertices.
  D3DDEV8.SetRenderState(D3DRS_FILLMODE, D3DFILL_WIREFRAME);
  D3DDEV8.DrawPrimitive(D3DPT_TRIANGLELIST, 0, FACENUM);

  // Render picked faces.
  if NumPickedTriangles > 0 then
    begin
      D3DDEV8.SetStreamSource(0, VertexBuffer2, sizeof(TCustomVertex));
      D3DDEV8.SetRenderState(D3DRS_FILLMODE, D3DFILL_SOLID);
      D3DDEV8.DrawPrimitive(D3DPT_TRIANGLELIST, 0, NumPickedTriangles);
    end;

  // End the scene.
  D3DDEV8.EndScene;

  // Display scene.
  D3DDEV8.Present(NIL, NIL, 0, NIL);
end;

// -------------------------------------------------------------------

procedure TForm1.AppOnIdle(Sender: TObject; var Done: Boolean);

begin
  Done := false;
  if (not DXReady) or (not WindowReady) then exit;
  RenderFrame(Handle);
  // Speed control
  while GetTickCount - FRAMECONTROL < FRAMEAT do Sleep(0);
  FRAMECONTROL := GetTickCount;
end;

// ------------------------------------------------------------------

procedure TForm1.AppOnActivate(Sender: TObject);

begin
  WindowReady := true;
end;

// ------------------------------------------------------------------

procedure TForm1.AppOnDeactivate(Sender: TObject);

begin
  WindowReady := false;
end;

// -------------------------------------------------------------------

procedure hHalt(_errMsg: pChar);

begin
  MessageBox(0, _errMsg, 'Error', MB_ICONERROR);
  Application.Terminate;
end;

// -------------------------------------------------------------------

function GetCustomVertex(_px, _py, _pz: single): TCustomVertex;

begin
  with Result do
   begin
     Position.x := _px;
     Position.y := _py;
     Position.z := _pz;
   end;
end;

// -------------------------------------------------------------------

function InitVertexBuffer: boolean;

var _vb  : array[0..11] of TCustomVertex;
    _ptr : pByte;

begin
  Result := false;

  // Initialize the vertices.
  _vb[ 0] := GetCustomVertex( -1.0, -1.0,  0.0);
  _vb[ 1] := GetCustomVertex(  1.0,  1.0,  0.0);
  _vb[ 2] := GetCustomVertex(  1.0, -1.0,  0.0);

  _vb[ 3] := GetCustomVertex( -1.0, -1.0,  0.0);
  _vb[ 4] := GetCustomVertex( -1.0,  1.0,  0.0);
  _vb[ 5] := GetCustomVertex(  1.0,  1.0,  0.0);

  _vb[ 6] := GetCustomVertex(  1.0, -1.0,  0.0);
  _vb[ 7] := GetCustomVertex(  1.0,  1.0,  0.0);
  _vb[ 8] := GetCustomVertex( -1.0, -1.0,  0.0);

  _vb[ 9] := GetCustomVertex(  1.0,  1.0,  0.0);
  _vb[10] := GetCustomVertex( -1.0,  1.0,  0.0);
  _vb[11] := GetCustomVertex( -1.0, -1.0,  0.0);

  // Create a vertex buffers.
  if failed(D3DDEV8.CreateVertexBuffer(SizeOf(_vb), 0,
                                       D3DFVF_CUSTOMVERTEX,
                                       D3DPOOL_DEFAULT,
                                       VertexBuffer1)) then exit;

  if failed(D3DDEV8.CreateVertexBuffer(SizeOf(_vb), 0,
                                       D3DFVF_CUSTOMVERTEX,
                                       D3DPOOL_DEFAULT,
                                       VertexBuffer2)) then exit;

  // Lock, fill and unlock vertex buffer 1.
  if failed(VertexBuffer1.Lock(0, SizeOf(_vb), _ptr, 0)) then exit;
  Move(_vb, _ptr^, SizeOf(_vb));
  VertexBuffer1.Unlock;

  // All right
  Result := true;
end;

// -------------------------------------------------------------------

procedure TForm1.FormActivate(Sender: TObject);

begin
  FRAMECONTROL := GetTickCount;

  ClientWidth  := DWIDTH;
  ClientHeight := DHEIGHT;

  // Initialize application events
  Application.OnIdle       := AppOnIdle;
  Application.OnActivate   := AppOnActivate;
  Application.OnDeactivate := AppOnDeactivate;
  Application.OnRestore    := AppOnActivate;
  Application.OnMinimize   := AppOnDeActivate;

  // Initialize Direct Graphics (first try with Hardware Accel. (HAL))
  if not InitDGfx(Handle, false, 0, 0, 0, 0, true, true) then
    begin
      MessageBox(0, WRN00, 'Warning.', MB_ICONWARNING);
      CloseDGfx;
      if not InitDGfx(Handle, false, 0, 0, 0, 0, false, true) then
        begin hHalt(ERR00); exit; end;
    end;

  // Initialize VertexBuffer
  if not InitVertexBuffer then begin hHalt(ERR01); exit; end;

  // Set Material And Light
  SetMaterial;

  // Enable lights.
  D3DDEV8.SetRenderState(D3DRS_LIGHTING, 1);

  // Set ambiental light.
  D3DDEV8.SetRenderState(D3DRS_AMBIENT, $FFFFFF);

  // All ok
  DXReady := true;
end;

// -------------------------------------------------------------------

procedure TForm1.FormDestroy(Sender: TObject);

begin
  DXReady := false;
  // Free Vertex Buffer
  VertexBuffer1 := NIL;
  VertexBuffer2 := NIL;
  // Free DX
  CloseDGfx;
end;

// -------------------------------------------------------------------

end.
